@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use vars qw($VERSION);
 
-$VERSION = '1.56_04';
+$VERSION = '1.56_15';
 
 =head1 NAME
 
@@ -15,8 +15,11 @@ App::Cpan - easily interact with CPAN from the command line
 	cpan module_name [ module_name ... ]
 
 	# with switches, installs modules with extra behavior
-	cpan [-cfimt] module_name [ module_name ... ]
+	cpan [-cfFimt] module_name [ module_name ... ]
 
+	# use local::lib
+	cpan -l module_name [ module_name ... ]
+	
 	# with just the dot, install from the distribution in the
 	# current directory
 	cpan .
@@ -25,7 +28,7 @@ App::Cpan - easily interact with CPAN from the command line
 	cpan
 
 	# without arguments, but some switches
-	cpan [-ahrvACDLO]
+	cpan [-ahruvACDLO]
 
 =head1 DESCRIPTION
 
@@ -33,26 +36,25 @@ This script provides a command interface (not a shell) to CPAN. At the
 moment it uses CPAN.pm to do the work, but it is not a one-shot command
 runner for CPAN.pm.
 
-=head2 Meta Options
-
-These options are mutually exclusive, and the script processes them in
-this order: [hvCAar].  Once the script finds one, it ignores the others,
-and then exits after it finishes the task.  The script ignores any other
-command line options.
+=head2 Options
 
 =over 4
 
 =item -a
 
-Creates the CPAN.pm autobundle with CPAN::Shell->autobundle.
+Creates a CPAN.pm autobundle with CPAN::Shell->autobundle.
 
 =item -A module [ module ... ]
 
-Shows the primary maintainers for the specified modules
+Shows the primary maintainers for the specified modules.
+
+=item -c module
+
+Runs a `make clean` in the specified module's directories.
 
 =item -C module [ module ... ]
 
-Show the C<Changes> files for the specified modules
+Show the F<Changes> files for the specified modules
 
 =item -D module [ module ... ]
 
@@ -61,6 +63,21 @@ Show the module details. This prints one line for each out-of-date module
 Each line has three columns: module name, local version, and CPAN
 version.
 
+=item -f
+
+Force the specified action, when it normally would have failed. Use this
+to install a module even if its tests fail. When you use this option,
+-i is not optional for installing a module when you need to force it:
+
+	% cpan -f -i Module::Foo
+
+=item -F
+
+Turn off CPAN.pm's attempts to lock anything. You should be careful with 
+this since you might end up with multiple scripts trying to muck in the
+same directory. This isn't so much of a concern if you're loading a special
+config with C<-j>, and that config sets up its own work directories.
+
 =item -g module [ module ... ]
 
 Downloads to the current directory the latest distribution of the module.
@@ -69,10 +86,22 @@ Downloads to the current directory the latest distribution of the module.
 
 UNIMPLEMENTED
 
-Downloads to the current directory the latest distribution of the
-module, unpack each distribution, and create a git repository for each
+Download to the current directory the latest distribution of the
+modules, unpack each distribution, and create a git repository for each
 distribution.
 
+If you want this feature, check out Yanick Champoux's C<Git::CPAN::Patch>
+distribution.
+
+=item -h
+
+Print a help message and exit. When you specify C<-h>, it ignores all
+of the other options and arguments.
+
+=item -i
+
+Install the specified modules.
+
 =item -j Config.pm
 
 Load the file that has the CPAN configuration data. This should have the
@@ -81,56 +110,42 @@ C<$CPAN::Config> as an anonymous hash.
 
 =item -J
 
-Dump the configuration in the same format that CPAN.pm uses.
+Dump the configuration in the same format that CPAN.pm uses. This is useful
+for checking the configuration as well as using the dump as a starting point
+for a new, custom configuration.
+
+=item -l
+
+Use C<local::lib>.
 
 =item -L author [ author ... ]
 
 List the modules by the specified authors.
 
-=item -h
+=item -m
 
-Prints a help message.
+Make the specified modules.
 
 =item -O
 
 Show the out-of-date modules.
 
+=item -t
+
+Run a `make test` on the specified modules.
+
 =item -r
 
 Recompiles dynamically loaded modules with CPAN::Shell->recompile.
 
+=item -u
+
+Upgrade all installed modules. Blindly doing this can really break things,
+so keep a backup.
+
 =item -v
 
-Print the script version and CPAN.pm version.
-
-=back
-
-=head2 Module options
-
-These options are mutually exclusive, and the script processes them in
-alphabetical order. It only processes the first one it finds.
-
-=over 4
-
-=item c
-
-Runs a `make clean` in the specified module's directories.
-
-=item f
-
-Forces the specified action, when it normally would have failed.
-
-=item i
-
-Installed the specified modules.
-
-=item m
-
-Makes the specified modules.
-
-=item t
-
-Runs a `make test` on the specified modules.
+Print the script version and CPAN.pm version then exit.
 
 =back
 
@@ -148,6 +163,9 @@ Runs a `make test` on the specified modules.
 	# recompile modules
 	cpan -r
 
+	# upgrade all installed modules
+	cpan -u
+
 	# install modules ( sole -i is optional )
 	cpan -i Netscape::Booksmarks Business::ISBN
 
@@ -164,6 +182,7 @@ Runs a `make test` on the specified modules.
 use autouse Carp => qw(carp croak cluck);
 use CPAN ();
 use autouse Cwd => qw(cwd);
+use autouse 'Data::Dumper' => qw(Dumper);
 use File::Spec::Functions;
 use File::Basename;
 
@@ -201,6 +220,7 @@ $Default = 'default';
 	'i'      => 'install',
 	'm'      => 'make',
 	't'      => 'test',
+	'u'      => 'upgrade',
 	);
 @CPAN_OPTIONS = grep { $_ ne $Default } sort keys %CPAN_METHODS;
 
@@ -210,32 +230,43 @@ $Default = 'default';
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
 # map switches to the subroutines in this script, along with other information.
 # use this stuff instead of hard-coded indices and values
+sub NO_ARGS   () { 0 }
+sub ARGS      () { 1 }
+sub GOOD_EXIT () { 0 }
+
 %Method_table = (
 # key => [ sub ref, takes args?, exit value, description ]
-	h =>  [ \&_print_help,        0, 0, 'Printing help'                ],
-	v =>  [ \&_print_version,     0, 0, 'Printing version'             ],
 
-	g =>  [ \&_download,          0, 0, 'Download the latest distro'   ],
-	G =>  [ \&_gitify,            0, 0, 'Down and gitify the latest distro' ],
-	j =>  [ \&_load_config,       1, 0, 'Use specified config file'    ],
-	J =>  [ \&_dump_config,       0, 0, 'Dump configuration to stdout' ],
+	# options that do their thing first, then exit
+	h =>  [ \&_print_help,        NO_ARGS, GOOD_EXIT, 'Printing help'                ],
+	v =>  [ \&_print_version,     NO_ARGS, GOOD_EXIT, 'Printing version'             ],
+
+	# options that affect other options
+	j =>  [ \&_load_config,          ARGS, GOOD_EXIT, 'Use specified config file'    ],
+	J =>  [ \&_dump_config,       NO_ARGS, GOOD_EXIT, 'Dump configuration to stdout' ],
+	F =>  [ \&_lock_lobotomy,     NO_ARGS, GOOD_EXIT, 'Turn off CPAN.pm lock files'  ],
+
+	# options that do their one thing
+	g =>  [ \&_download,          NO_ARGS, GOOD_EXIT, 'Download the latest distro'        ],
+	G =>  [ \&_gitify,            NO_ARGS, GOOD_EXIT, 'Down and gitify the latest distro' ],
 	
-	C =>  [ \&_show_Changes,      1, 0, 'Showing Changes file'         ],
-	A =>  [ \&_show_Author,       1, 0, 'Showing Author'               ],
-	D =>  [ \&_show_Details,      1, 0, 'Showing Details'              ],
-	O =>  [ \&_show_out_of_date,  0, 0, 'Showing Out of date'          ],
+	C =>  [ \&_show_Changes,         ARGS, GOOD_EXIT, 'Showing Changes file'         ],
+	A =>  [ \&_show_Author,          ARGS, GOOD_EXIT, 'Showing Author'               ],
+	D =>  [ \&_show_Details,         ARGS, GOOD_EXIT, 'Showing Details'              ],
+	O =>  [ \&_show_out_of_date,  NO_ARGS, GOOD_EXIT, 'Showing Out of date'          ],
 
-	l =>  [ \&_list_all_mods,     0, 0, 'Listing all modules'          ],
+	l =>  [ \&_list_all_mods,     NO_ARGS, GOOD_EXIT, 'Listing all modules'          ],
 
-	L =>  [ \&_show_author_mods,  1, 0, 'Showing author mods'          ],
-	a =>  [ \&_create_autobundle, 0, 0, 'Creating autobundle'          ],
-	r =>  [ \&_recompile,         0, 0, 'Recompiling'                  ],
+	L =>  [ \&_show_author_mods,     ARGS, GOOD_EXIT, 'Showing author mods'          ],
+	a =>  [ \&_create_autobundle, NO_ARGS, GOOD_EXIT, 'Creating autobundle'          ],
+	r =>  [ \&_recompile,         NO_ARGS, GOOD_EXIT, 'Recompiling'                  ],
+	u =>  [ \&_upgrade,           NO_ARGS, GOOD_EXIT, 'Running `make test`'          ],
 
-	c =>  [ \&_default,           1, 0, 'Running `make clean`'         ],
-	f =>  [ \&_default,           1, 0, 'Installing with force'        ],
-	i =>  [ \&_default,           1, 0, 'Running `make install`'       ],
-   'm' => [ \&_default,           1, 0, 'Running `make`'               ],
-	t =>  [ \&_default,           1, 0, 'Running `make test`'          ],
+	c =>  [ \&_default,              ARGS, GOOD_EXIT, 'Running `make clean`'         ],
+	f =>  [ \&_default,              ARGS, GOOD_EXIT, 'Installing with force'        ],
+	i =>  [ \&_default,              ARGS, GOOD_EXIT, 'Running `make install`'       ],
+   'm' => [ \&_default,              ARGS, GOOD_EXIT, 'Running `make`'               ],
+	t =>  [ \&_default,              ARGS, GOOD_EXIT, 'Running `make test`'          ],
 
 	);
 
@@ -288,6 +319,12 @@ sub _process_setup_options
 			);
 		}
 		
+	if( $options->{F} )
+		{
+		$Method_table{F}[ $Method_table_index{code} ]->( $options->{F} );
+		delete $options->{F};
+		}
+
 	my $option_count = grep { $options->{$_} } @option_order;
 	no warnings 'uninitialized';
 	$option_count -= $options->{'f'}; # don't count force
@@ -312,20 +349,23 @@ my $logger;
 sub run
 	{
 	my $class = shift;
-	
+
 	my $return_value = HEY_IT_WORKED; # assume that things will work
-	
+
 	$logger = $class->_init_logger;
 	$logger->debug( "Using logger from @{[ref $logger]}" );
-	
+
 	$class->_hook_into_CPANpm_report;
-	
+	$logger->debug( "Hooked into output" );
+
 	$class->_stupid_interface_hack_for_non_rtfmers;
-	
+	$logger->debug( "Patched cargo culting" );
+
 	my $options = $class->_process_options;
-	
+	$logger->debug( "Options are @{[Dumper($options)]}" );
+
 	$class->_process_setup_options( $options );
-	
+
 	OPTION: foreach my $option ( @option_order )
 		{	
 		next unless $options->{$option};
@@ -333,18 +373,18 @@ sub run
 		my( $sub, $takes_args, $description ) = 
 			map { $Method_table{$option}[ $Method_table_index{$_} ] }
 			qw( code takes_args );
-			
+
 		unless( ref $sub eq ref sub {} )
 			{
 			$return_value = THE_PROGRAMMERS_AN_IDIOT;
 			last OPTION;
 			}
-		
+
 		$logger->info( "$description -- ignoring other arguments" )
 			if( @ARGV && ! $takes_args );
-			
-		$return_value = $sub->( \ @ARGV, $options );
 		
+		$return_value = $sub->( \ @ARGV, $options );
+
 		last;
 		}
 
@@ -363,9 +403,13 @@ sub _init_logger
 	{
 	my $log4perl_loaded = eval "require Log::Log4perl; 1";
 	
-	return Local::Null::Logger->new unless $log4perl_loaded;
+    unless( $log4perl_loaded )
+        {
+        $logger = Local::Null::Logger->new;
+        return $logger;
+        }
 	
-	my $LEVEL = 'INFO';
+	my $LEVEL = $ENV{CPANSCRIPT_LOGLEVEL} || 'INFO';
 	
 	Log::Log4perl::init( \ <<"HERE" );
 log4perl.rootLogger=$LEVEL, A1
@@ -374,7 +418,7 @@ log4perl.appender.A1.layout=PatternLayout
 log4perl.appender.A1.layout.ConversionPattern=%m%n
 HERE
 	
-	my $logger = Log::Log4perl->get_logger( 'App::Cpan' );
+	$logger = Log::Log4perl->get_logger( 'App::Cpan' );
 	}
 	
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
@@ -447,16 +491,16 @@ sub _hook_into_CPANpm_report
 	no warnings 'redefine';
 	
 	*CPAN::Shell::myprint = sub {
-    	my($self,$what) = @_;
+		my($self,$what) = @_;
 		$scalar .= $what;
-    	$self->print_ornamented($what,
+		$self->print_ornamented($what,
 			$CPAN::Config->{colorize_print}||'bold blue on_white',
 			);
 		};
 
 	*CPAN::Shell::mywarn = sub {
 		my($self,$what) = @_;
-		$scalar .= $what;
+		$scalar .= $what;   
 		$self->print_ornamented($what, 
 			$CPAN::Config->{colorize_warn}||'bold red on_white'
 			);
@@ -468,19 +512,52 @@ sub _clear_cpanpm_output { $scalar = '' }
 	
 sub _get_cpanpm_output   { $scalar }
 
+BEGIN {
+my @skip_lines = (
+	qr/^\QWarning \(usually harmless\)/,
+	qr/\bwill not store persistent state\b/,
+	qr(//hint//),
+	qr/^\s+reports\s+/,
+	);
+
 sub _get_cpanpm_last_line
 	{
 	open my($fh), "<", \ $scalar;
-	( <$fh> )[-1];
+	
+	my @lines = <$fh>;
+	
+    # This is a bit ugly. Once we examine a line, we have to
+    # examine the line before it and go through all of the same
+    # regexes. I could do something fancy, but this works.
+    REGEXES: {
+	foreach my $regex ( @skip_lines )
+		{
+		if( $lines[-1] =~ m/$regex/ )
+            {
+            pop @lines;
+            redo REGEXES; # we have to go through all of them for every line!
+            }
+		}
+    }
+    
+    $logger->debug( "Last interesting line of CPAN.pm output is:\n\t$lines[-1]" );
+    
+	$lines[-1];
 	}
+}
 
+BEGIN {
+my $epic_fail_words = join '|',
+	qw( Error stop(?:ping)? problems force not unsupported fail(?:ed)? );
+	
 sub _cpanpm_output_indicates_failure
 	{
 	my $last_line = _get_cpanpm_last_line();
 	
-	my $result = $last_line =~ /\b(?:Error|stop|problems?|force|not|unsupported|fail)\b/i;
+	my $result = $last_line =~ /\b(?:$epic_fail_words)\b/i;
 	$result || ();
 	}
+}
 	
 sub _cpanpm_output_indicates_success
 	{
@@ -526,7 +603,7 @@ sub _create_autobundle
 	return HEY_IT_WORKED;
 	}
 
-sub _recompiling
+sub _recompile
 	{
 	$logger->info( "Recompiling dynamically-loaded extensions" );
 
@@ -535,6 +612,15 @@ sub _recompiling
 	return HEY_IT_WORKED;
 	}
 
+sub _upgrade
+	{
+	$logger->info( "Upgrading all modules" );
+
+	CPAN::Shell->upgrade();
+
+	return HEY_IT_WORKED;
+	}
+
 sub _load_config # -j
 	{	
 	my $file = shift || '';
@@ -574,6 +660,16 @@ sub _dump_config
 	return HEY_IT_WORKED;
 	}
 
+sub _lock_lobotomy
+	{
+	no warnings 'redefine';
+	
+	*CPAN::_flock    = sub { 1 };
+	*CPAN::checklock = sub { 1 };
+
+	return HEY_IT_WORKED;
+	}
+	
 sub _download
 	{	
 	my $args = shift;
@@ -802,20 +898,21 @@ sub _list_all_mods
 	
 	my $args = shift;
 	
-	my( $wanted, $reporter ) = _generator();
 	
 	my $fh = \*STDOUT;
 	
-	foreach my $inc ( @INC )
+	INC: foreach my $inc ( @INC )
 		{		
+		my( $wanted, $reporter ) = _generator();
 		File::Find::find( { wanted => $wanted }, $inc );
 		
 		my $count = 0;
-		foreach my $file ( @{ $reporter->() } )
+		FILE: foreach my $file ( @{ $reporter->() } )
 			{
 			my $version = _parse_version_safely( $file );
 			
 			my $module_name = _path_to_module( $inc, $file );
+			next FILE unless defined $module_name;
 			
 			print $fh "$module_name\t$version\n";
 			
@@ -891,6 +988,7 @@ sub _eval_version
 sub _path_to_module
 	{
 	my( $inc, $path ) = @_;
+	return if length $path< length $inc;
 	
 	my $module_path = substr( $path, length $inc );
 	$module_path =~ s/\.pm\z//;
@@ -963,7 +1061,7 @@ brian d foy, C<< <bdfoy@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2008, brian d foy, All Rights Reserved.
+Copyright (c) 2001-2009, brian d foy, All Rights Reserved.
 
 You may redistribute this under the same terms as Perl itself.
 
